@@ -49,6 +49,7 @@ typedef enum {
 	EXIF_CATEGORY_CAMERA,
 	EXIF_CATEGORY_IMAGE_DATA,
 	EXIF_CATEGORY_IMAGE_TAKING_CONDITIONS,
+	EXIF_CATEGORY_GPS_DATA,
 	EXIF_CATEGORY_MAKER_NOTE,
 	EXIF_CATEGORY_OTHER,
 #ifdef HAVE_EXEMPI
@@ -68,13 +69,14 @@ static ExifCategoryInfo exif_categories[] = {
 	{ N_("Camera"),                  "0" },
 	{ N_("Image Data"),              "1" },
 	{ N_("Image Taking Conditions"), "2" },
-	{ N_("Maker Note"),              "3" },
-	{ N_("Other"),                   "4" },
+	{ N_("GPS Data"),                "3" },
+	{ N_("Maker Note"),              "4" },
+	{ N_("Other"),                   "5" },
 #ifdef HAVE_EXEMPI
-	{ N_("XMP Exif"),                "5" },
-	{ N_("XMP IPTC"),                "6" },
-	{ N_("XMP Rights Management"),   "7" },
-	{ N_("XMP Other"),               "8" },
+	{ N_("XMP Exif"),                "6" },
+	{ N_("XMP IPTC"),                "7" },
+	{ N_("XMP Rights Management"),   "8" },
+	{ N_("XMP Other"),               "9" },
 #endif
 	{ NULL, NULL }
 };
@@ -255,15 +257,14 @@ eom_exif_details_init (EomExifDetails *exif_details)
 
 	/* Value column */
 	cell = gtk_cell_renderer_text_new ();
-        column = gtk_tree_view_column_new_with_attributes (_("Value"), cell,
-                                                           "text", MODEL_COLUMN_VALUE,
-							    NULL);
+	g_object_set (cell,
+	              "editable", TRUE,
+	              NULL);
+	column = gtk_tree_view_column_new_with_attributes (_("Value"), cell,
+	                                                   "text", MODEL_COLUMN_VALUE,
+	                                                   NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (exif_details), column);
 
-#if !GTK_CHECK_VERSION (3, 14, 0)
-	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (exif_details), TRUE);
-#endif
-
 	eom_exif_details_reset (exif_details);
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (exif_details),
@@ -287,6 +288,11 @@ get_exif_category (ExifEntry *entry)
 	ExifCategory cat = EXIF_CATEGORY_OTHER;
 	int i;
 
+	/* Some GPS tag IDs overlap with other ones, so check the IFD */
+	if (exif_entry_get_ifd (entry) == EXIF_IFD_GPS) {
+		return EXIF_CATEGORY_GPS_DATA;
+	}
+
 	for (i = 0; exif_tag_category_map [i].id != -1; i++) {
 		if (exif_tag_category_map[i].id == (int) entry->tag) {
 			cat = exif_tag_category_map[i].category;
@@ -350,6 +356,94 @@ set_row_data (GtkTreeStore *store, char *path, char *parent, const char *attribu
 }
 
 #ifdef HAVE_EXIF
+
+static const char *
+eom_exif_entry_get_value (ExifEntry    *e,
+                          char         *buf,
+                          guint         n_buf)
+{
+	ExifByteOrder bo;
+
+	/* For now we only want to reformat some GPS values */
+	if (G_LIKELY (exif_entry_get_ifd (e) != EXIF_IFD_GPS))
+		return exif_entry_get_value (e, buf, n_buf);
+
+	bo = exif_data_get_byte_order (e->parent->parent);
+
+	/* Cast to number to avoid warnings about values not in enumeration */
+	switch ((guint16) e->tag) {
+		case EXIF_TAG_GPS_LATITUDE:
+		case EXIF_TAG_GPS_LONGITUDE:
+		{
+			gsize rational_size;
+			ExifRational r;
+			gfloat h = 0., m = 0.;
+
+
+			rational_size = exif_format_get_size (EXIF_FORMAT_RATIONAL);
+			if (G_UNLIKELY (e->components != 3 ||
+			    e->format != EXIF_FORMAT_RATIONAL))
+				return exif_entry_get_value (e, buf, n_buf);
+
+			r = exif_get_rational (e->data, bo);
+			if (r.denominator != 0)
+				h = (gfloat)r.numerator / r.denominator;
+
+			r = exif_get_rational (e->data + rational_size, bo);
+			if (r.denominator != 0)
+				m = (gfloat)r.numerator / (gfloat)r.denominator;
+
+			r = exif_get_rational (e->data + (2 * rational_size),
+					       bo);
+			if (r.numerator != 0 && r.denominator != 0) {
+				gfloat s;
+
+				s = (gfloat)r.numerator / (gfloat)r.denominator;
+				g_snprintf (buf, n_buf,
+					    "%.0f° %.0f' %.2f\"",
+					    h, m, s);
+			} else {
+				g_snprintf (buf, n_buf,
+					    "%.0f° %.2f'",
+					    h, m);
+			}
+
+			break;
+		}
+		case EXIF_TAG_GPS_LATITUDE_REF:
+		case EXIF_TAG_GPS_LONGITUDE_REF:
+		{
+			if (G_UNLIKELY (e->components != 2 ||
+			    e->format != EXIF_FORMAT_ASCII))
+				return exif_entry_get_value (e, buf, n_buf);
+
+			switch (e->data[0]) {
+			case 'N':
+				g_snprintf (buf, n_buf, "%s", _("North"));
+				break;
+			case 'E':
+				g_snprintf (buf, n_buf, "%s", _("East"));
+				break;
+			case 'W':
+				g_snprintf (buf, n_buf, "%s", _("West"));
+				break;
+			case 'S':
+				g_snprintf (buf, n_buf, "%s", _("South"));
+				break;
+			default:
+				return exif_entry_get_value (e, buf, n_buf);
+				break;
+			}
+			break;
+		}
+		default:
+			return exif_entry_get_value (e, buf, n_buf);
+			break;
+	}
+
+	return buf;
+}
+
 static void
 exif_entry_cb (ExifEntry *entry, gpointer data)
 {
@@ -357,22 +451,32 @@ exif_entry_cb (ExifEntry *entry, gpointer data)
 	EomExifDetails *view;
 	EomExifDetailsPrivate *priv;
 	ExifCategory cat;
+	ExifIfd ifd = exif_entry_get_ifd (entry);
 	char *path;
 	char b[1024];
+	const gint key = ifd << 16 | entry->tag;
+
+	/* This should optimize away if comparision is correct */
+	g_warn_if_fail (EXIF_IFD_COUNT <= G_MAXUINT16);
 
 	view = EOM_EXIF_DETAILS (data);
 	priv = view->priv;
 
 	store = GTK_TREE_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (view)));
 
-	path = g_hash_table_lookup (priv->id_path_hash, GINT_TO_POINTER (entry->tag));
+	/* Take the tag's IFD into account when caching their GtkTreePaths.
+	 * That should fix key collisions for tags that have the same number
+	 * but are stored in different IFDs. Exif tag numbers are 16-bit
+	 * values so we should be able to set the high word to the IFD number.
+	 */
+	path = g_hash_table_lookup (priv->id_path_hash, GINT_TO_POINTER (key));
 
 	if (path != NULL) {
 		set_row_data (store,
 			      path,
 			      NULL,
-			      exif_tag_get_name (entry->tag),
-			      exif_entry_get_value (entry, b, sizeof(b)));
+			      exif_tag_get_name_in_ifd (entry->tag, ifd),
+			      eom_exif_entry_get_value (entry, b, sizeof(b)));
 	} else {
 
 		ExifMnoteData *mnote = (entry->tag == EXIF_TAG_MAKER_NOTE ?
@@ -403,13 +507,13 @@ exif_entry_cb (ExifEntry *entry, gpointer data)
 			path = set_row_data (store,
 					     NULL,
 					     exif_categories[cat].path,
-					     exif_tag_get_name (entry->tag),
-					     exif_entry_get_value (entry, b,
-								   sizeof(b)));
+					     exif_tag_get_name_in_ifd (entry->tag, ifd),
+					     eom_exif_entry_get_value (entry, b,
+						                           sizeof(b)));
 
 			g_hash_table_insert (priv->id_path_hash,
-					     GINT_TO_POINTER (entry->tag),
-					     path);
+			                     GINT_TO_POINTER (key),
+			                     path);
 		}
 	}
 }
